<?php

namespace NoccyLabs\PulseAudio\Sink;

/**
 * Record from a PulseAudio sink
 * 
 */
class Recorder
{
    /** @const The number of seconds to wait for parec to exit */
    const STOP_TIMEOUT = 5;
    /** @var Sink The sink to record from */
    protected $sink;
    /** @var resource The process handle of parec */
    protected $proc;
    /** @var array|null The pipes of the sink */
    protected $pipes;

    /**
     * Constructor
     *
     * @param Sink $sink The sink to record from
     */
    public function __construct(Sink $sink)
    {
        $this->sink = $sink;
    }

    /**
     * Set the filename to write to
     *
     * @param string $filename
     */
    public function setFilename($filename)
    {
        $this->filename = $filename;
    }

    /**
     * Start recording.
     *
     * @return bool True if started, false if a capture is already running
     */
    public function start()
    {
        // Don't do anything if we are already recording'
        if ($this->proc) {
            $status = proc_get_status($this->proc);
            if ($status['running']==true) {
                return false;
            }
            proc_close($this->proc);
        }
        // Otherwise start recording
        $cmdl = sprintf(
            "parecord --format=s16le --file-format=wav --monitor-stream %d", 
            $this->sink->getMonitorIndex()
        );

        $filename = $this->filename?:"output.wav";

        echo "$ {$cmdl}\n";

        $ds = [
            0 => STDIN,
            1 => fopen($filename,'wb'),
            2 => STDERR
        ];
        $this->proc = proc_open($cmdl, $ds, $pipes);
        $this->pipes = $pipes;
    }

    /**
     * Stop recording.
     *
     * @return bool True if not capturing or capture stopped ok
     */    
    public function stop()
    {
        if (!$this->proc) {
            return true;
        }
        
        // Send a SIGINT to parec
        proc_terminate($this->proc, 2);
        
        // Wait for parec to close
        $expire = microtime(true)+self::STOP_TIMEOUT;
        $this->pipes = null;
        do {
            $status = proc_get_status($this->proc);
            usleep(10000);
        } while ($status['running'] && (microtime(true)<$expire));
        
        // Return status; if running return false
        return !$status['running'];
    }


}

